#include "Calculator_Demo_objects.h"
#include "Calculator_Demo_resources.h"
#include "built_in.h"
#include "FT810_Types.h"

sbit FT810_RST at LATJ6_bit;
sbit FT810_CS at LATJ4_bit;
sbit FT810_RST_Direction at TRISJ6_bit;
sbit FT810_CS_Direction at TRISJ4_bit;

// Object type constants
// Usage: VTFT stack internally
const VTFT_OT_CBUTTON      = 1;
const VTFT_OT_CBUTTONROUND = 3;
const VTFT_OT_LABEL        = 4;
const VTFT_OT_CLABEL       = 5;
const VTFT_OT_CBOX         = 13;
// ~Object type constants

// Event type constants
// Usage: OnEvent
const VTFT_EVT_UP    = 0;
const VTFT_EVT_DOWN  = 1;
const VTFT_EVT_CLICK = 2;
const VTFT_EVT_PRESS = 3;
// ~Event type constants

// Sound action constants
// Usage: sound event action property and ProcessEvent
const VTFT_SNDACT_NONE  = 0;
const VTFT_SNDACT_PLAY  = 1;
const VTFT_SNDACT_STOP  = 2;
// ~Sound action constants

// Resource loading constants.
// Usage: DrawScreenO and LoadCurrentScreenResToGRAM
const VTFT_LOAD_RES_NONE    = 0x00; // do not load g-ram resources
const VTFT_LOAD_RES_STATIC  = 0x01; // load g-ram resources for static objects
const VTFT_LOAD_RES_DYNAMIC = 0x02; // load g-ram resources for dynamic objects
const VTFT_LOAD_RES_ALL     = 0x03; // load g-ram resources for all objects
// ~Resource loading constants

// Display effect constants
// Usage: DrawScreenO
const VTFT_DISPLAY_EFF_NONE         = 0x00; // no effect when switching between screens
const VTFT_DISPLAY_EFF_LIGHTS_FADE  = 0x04; // backlight: fade out before, fade in after drawing new screen
const VTFT_DISPLAY_EFF_LIGHTS_OFF   = 0x08; // backlight: turn off before, turn on after drawing new screen
// ~Display effect constants

// Stack flags
// Usage: internally used by VTFT stack
const VTFT_INT_REPAINT_ON_DOWN     = 1 << 0;
const VTFT_INT_REPAINT_ON_UP       = 1 << 1;
const VTFT_INT_BRING_TO_FRONT      = 1 << 2;
const VTFT_INT_INTRINSIC_CLICK_EFF = 1 << 3;
// ~Stack flags

// Table of object draw handlers
// Use object type constants to access coresponding object draw handler
const TPointer DrawHandlerTable[44] = {
  0,                 // Button draw handler not used
  &DrawCButton,      // CButton draw handler
  0,                 // ButtonRound draw handler not used
  &DrawCButtonRound, // CButtonRound draw handler
  &DrawLabel,        // Label draw handler
  &DrawCLabel,       // CLabel draw handler
  0,                 // Image draw handler not used
  0,                 // CImage draw handler not used
  0,                 // Circle draw handler not used
  0,                 // CCircle draw handler not used
  0,                 // CircleButton draw handler not used
  0,                 // CCircleButton draw handler not used
  0,                 // Box draw handler not used
  &DrawCBox,         // CBox draw handler
  0,                 // BoxRound draw handler not used
  0,                 // CBoxRound draw handler not used
  0,                 // Line draw handler not used
  0,                 // CLine draw handler not used
  0,                 // Polygon draw handler not used
  0,                 // CPolygon draw handler not used
  0,                 // CheckBox draw handler not used
  0,                 // RadioButton draw handler not used
  0,                 // ProgressBar draw handler not used
  0,                 // Audio draw handler not used
  0,                 // EveClock draw handler not used
  0,                 // EveGauge draw handler not used
  0,                 // EveDial draw handler not used
  0,                 // EveKeys draw handler not used
  0,                 // CEveKeys draw handler not used
  0,                 // EveProgressBar draw handler not used
  0,                 // EveScrollBar draw handler not used
  0,                 // EveToggle draw handler not used
  0,                 // EveSlider draw handler not used
  0,                 // EveSpinner draw handler not used
  0,                 // EveScreenSaver draw handler not used
  0,                 // EveSketch draw handler not used
  0,                 // EveButton draw handler not used
  0,                 // CEveButton draw handler not used
  0,                 // EveGradient draw handler not used
  0,                 // CEveGradient draw handler not used
  0,                 // EveText draw handler not used
  0,                 // CEveText draw handler not used
  0,                 // EveNumber draw handler not used
  0                  // CEveNumber draw handler not used
};
// ~Table of draw handler pointers


// Default configuration parameters
const TFT810Display VTFT_FT810_CONFIG_DISPLAY =
{
  48000000,        // Frequency          = main clock frequency
  0,               // OutRenderMode      = 0 normal, 1 write, 2 read
  0,               // RenderReadScanLine = scanline for read render mode
  0,               // RenderWriteTrigger = trigger for write render mode (read only)
  525,             // hCycle             = number if horizontal cycles for display
  43,              // hOffset            = horizontal offset from starting signal
  480,             // hSize              = width resolution
  0,               // hSync0             = hsync falls
  41,              // hSync1             = hsync rise
  286,             // vCycle             = number of vertical cycles for display
  12,              // vOffset            = vertical offset from start signal
  272,             // vSize              = height resolution
  0,               // vSync0             = vsync falls
  10,              // vSync1             = vsync rise
  0,               // Rotate             = rotate display
  0x01B6,          // OutBits            = output bits resolution
  0,               // OutDither          = output number of bits
  0x0000,          // OutSwizzle         = output swizzle
  0,               // OutCSpread         = output clock spread enable
  1,               // PClockPolarity     = clock polarity: 0 - rising edge, 1 - falling edge
  5,               // PClock             = clock prescaler of FT810: - 0 means disable and >0 means 48MHz/pclock
};

const TFT810Touch VTFT_FT810_CONFIG_TOUCH =
{
  3,               // TouchMode        = touch screen mode (2 bits): 0 - off, 1 - oneshot, 2 - frame, 3 - continuous
  1,               // TouchADCMode     = touch screen adc mode (1 bit): 0 - singleended, 1 - differential
  6000,            // TouchCharge      = Touchscreen charge time, units of 6 clocks
  3,               // TouchSettle      = touch screen settle time - 4 bits
  7,               // TouchOversample  = touch screen oversample - 4 bits
  2000,            // TouchRZThreshold = Touchscreen resistance threshold
};

const TFT810GPIO VTFT_FT810_CONFIG_GPIO =
{
  0xFFF0,          // GPIODIR = GPIO direction: 1 - output, 0 - input (16bit wide)
  0xFFFF,          // GPIO    = GPIO data latch
};

const TFT810PWM VTFT_FT810_CONFIG_PWM =
{
  255,             // Freq = PWM frequency - 14 bits
  127,             // Duty = PWM duty cycle, 0 to 128 is the range
};

const TFT810Interrupt VTFT_FT810_CONFIG_INTERRUPT =
{
  0,               // Flags  = interrupt flags (read only)
  0,               // Enable = global interrupt enable: 1 - enabled, 0 - disabled
  255,             // Mask   = interrupt mask value (individual interrupt enable): 1 - masked/disabled, 0 - enabled
};

const TFT810Sound VTFT_FT810_CONFIG_SOUND =
{
  0,               // Volume
  {0,              // Effect
  0},              // Pitch
  0,               // Play
};

const TFT810Audio VTFT_FT810_CONFIG_AUDIO =
{
  0,               // Volume
  0,               // StartAddress
  0,               // Length
  0,               // ReadPtr
  8000,            // Frequency
  0,               // Format
  0,               // Loop = audio playback mode
  0,               // Play
};

// Global variables

TTouchStat TouchS = {0};


/////////////////////////
TScreen *CurrentScreen = 0;


TScreen Calc_Demo;

TCButton       Button1 = {
  &Calc_Demo,           // Button1.OwnerScreen
  0,                    // Button1.Order
  1,                    // Button1.Visible
  255,                  // Button1.Opacity
  255,                  // Button1.Tag
  0,                    // Button1.Left
  0,                    // Button1.Top
  480,                  // Button1.Width
  30,                   // Button1.Height
  1,                    // Button1.Pen_Width
  0x0000,               // Button1.Pen_Color
  0x39E7,               // Button1.Color
  0xC618,               // Button1.Press_Color
  0xFFFF,               // Button1.ColorTo
  0xC618,               // Button1.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button1.Gradient
  Button1_Caption,      // Button1.Caption
  taCenter,             // Button1.TextAlign
  Tahoma_25x25_Bold,    // Button1.FontName
  0xFFFF,               // Button1.Font_Color
  1,                    // Button1.FontHandle
  0,                    // Button1.Source
  0,                    // Button1.Active
  0,                    // Button1.OnUp
  0,                    // Button1.OnDown
  0,                    // Button1.OnClick
  0                     // Button1.OnPress
};
const code char Button1_Caption[16] = "Calculator Demo";
TCBox          Box1 = {
  &Calc_Demo,           // Box1.OwnerScreen
  1,                    // Box1.Order
  1,                    // Box1.Visible
  255,                  // Box1.Opacity
  255,                  // Box1.Tag
  0,                    // Box1.Left
  30,                   // Box1.Top
  480,                  // Box1.Width
  107,                  // Box1.Height
  1,                    // Box1.Pen_Width
  0x0000,               // Box1.Pen_Color
  0x0720,               // Box1.Color
  0xE71C,               // Box1.Press_Color
  0xFFFF,               // Box1.ColorTo
  0xC618,               // Box1.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Box1.Gradient
  0,                    // Box1.Active
  0,                    // Box1.OnUp
  0,                    // Box1.OnDown
  0,                    // Box1.OnClick
  0                     // Box1.OnPress
};
TCButton       Button2 = {
  &Calc_Demo,           // Button2.OwnerScreen
  2,                    // Button2.Order
  1,                    // Button2.Visible
  255,                  // Button2.Opacity
  255,                  // Button2.Tag
  1,                    // Button2.Left
  137,                  // Button2.Top
  95,                   // Button2.Width
  34,                   // Button2.Height
  1,                    // Button2.Pen_Width
  0x8410,               // Button2.Pen_Color
  0x39E7,               // Button2.Color
  0xFC08,               // Button2.Press_Color
  0xFFFF,               // Button2.ColorTo
  0xC618,               // Button2.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button2.Gradient
  Button2_Caption,      // Button2.Caption
  taCenter,             // Button2.TextAlign
  Tahoma_25x25_Bold,    // Button2.FontName
  0xFFFF,               // Button2.Font_Color
  1,                    // Button2.FontHandle
  0,                    // Button2.Source
  1,                    // Button2.Active
  0,                    // Button2.OnUp
  0,                    // Button2.OnDown
  &Button2_OnClick,     // Button2.OnClick
  0                     // Button2.OnPress
};
TCEvent Button2_OnClick = {

  Button2OnClick,        // Button2_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button2_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button2_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button2_OnClick.Sound.Pitch
  255                    // Button2_OnClick.Sound.Volume
};
const code char Button2_Caption[2] = "7";
TCButton       Button3 = {
  &Calc_Demo,           // Button3.OwnerScreen
  3,                    // Button3.Order
  1,                    // Button3.Visible
  255,                  // Button3.Opacity
  255,                  // Button3.Tag
  95,                   // Button3.Left
  137,                  // Button3.Top
  95,                   // Button3.Width
  34,                   // Button3.Height
  1,                    // Button3.Pen_Width
  0x8410,               // Button3.Pen_Color
  0x39E7,               // Button3.Color
  0xFC08,               // Button3.Press_Color
  0xFFFF,               // Button3.ColorTo
  0xC618,               // Button3.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button3.Gradient
  Button3_Caption,      // Button3.Caption
  taCenter,             // Button3.TextAlign
  Tahoma_25x25_Bold,    // Button3.FontName
  0xFFFF,               // Button3.Font_Color
  1,                    // Button3.FontHandle
  0,                    // Button3.Source
  1,                    // Button3.Active
  0,                    // Button3.OnUp
  0,                    // Button3.OnDown
  &Button3_OnClick,     // Button3.OnClick
  0                     // Button3.OnPress
};
TCEvent Button3_OnClick = {

  Button3OnClick,        // Button3_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button3_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button3_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button3_OnClick.Sound.Pitch
  255                    // Button3_OnClick.Sound.Volume
};
const code char Button3_Caption[2] = "8";
TCButton       Button4 = {
  &Calc_Demo,           // Button4.OwnerScreen
  4,                    // Button4.Order
  1,                    // Button4.Visible
  255,                  // Button4.Opacity
  255,                  // Button4.Tag
  285,                  // Button4.Left
  137,                  // Button4.Top
  97,                   // Button4.Width
  34,                   // Button4.Height
  1,                    // Button4.Pen_Width
  0x8410,               // Button4.Pen_Color
  0x39E7,               // Button4.Color
  0xFC08,               // Button4.Press_Color
  0xFFFF,               // Button4.ColorTo
  0xC618,               // Button4.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button4.Gradient
  Button4_Caption,      // Button4.Caption
  taCenter,             // Button4.TextAlign
  Tahoma_25x25_Bold,    // Button4.FontName
  0xFFFF,               // Button4.Font_Color
  1,                    // Button4.FontHandle
  0,                    // Button4.Source
  1,                    // Button4.Active
  0,                    // Button4.OnUp
  0,                    // Button4.OnDown
  &Button4_OnClick,     // Button4.OnClick
  0                     // Button4.OnPress
};
TCEvent Button4_OnClick = {

  Button4OnClick,        // Button4_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button4_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button4_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button4_OnClick.Sound.Pitch
  255                    // Button4_OnClick.Sound.Volume
};
const code char Button4_Caption[2] = "/";
TCButton       Button5 = {
  &Calc_Demo,           // Button5.OwnerScreen
  5,                    // Button5.Order
  1,                    // Button5.Visible
  255,                  // Button5.Opacity
  255,                  // Button5.Tag
  190,                  // Button5.Left
  137,                  // Button5.Top
  95,                   // Button5.Width
  34,                   // Button5.Height
  1,                    // Button5.Pen_Width
  0x8410,               // Button5.Pen_Color
  0x39E7,               // Button5.Color
  0xFC08,               // Button5.Press_Color
  0xFFFF,               // Button5.ColorTo
  0xC618,               // Button5.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button5.Gradient
  Button5_Caption,      // Button5.Caption
  taCenter,             // Button5.TextAlign
  Tahoma_25x25_Bold,    // Button5.FontName
  0xFFFF,               // Button5.Font_Color
  1,                    // Button5.FontHandle
  0,                    // Button5.Source
  1,                    // Button5.Active
  0,                    // Button5.OnUp
  0,                    // Button5.OnDown
  &Button5_OnClick,     // Button5.OnClick
  0                     // Button5.OnPress
};
TCEvent Button5_OnClick = {

  Button5OnClick,        // Button5_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button5_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button5_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button5_OnClick.Sound.Pitch
  255                    // Button5_OnClick.Sound.Volume
};
const code char Button5_Caption[2] = "9";
TCButton       Button6 = {
  &Calc_Demo,           // Button6.OwnerScreen
  6,                    // Button6.Order
  1,                    // Button6.Visible
  255,                  // Button6.Opacity
  255,                  // Button6.Tag
  382,                  // Button6.Left
  137,                  // Button6.Top
  98,                   // Button6.Width
  34,                   // Button6.Height
  1,                    // Button6.Pen_Width
  0x8410,               // Button6.Pen_Color
  0x39E7,               // Button6.Color
  0xFC08,               // Button6.Press_Color
  0xFFFF,               // Button6.ColorTo
  0xC618,               // Button6.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button6.Gradient
  Button6_Caption,      // Button6.Caption
  taCenter,             // Button6.TextAlign
  Tahoma_25x25_Bold,    // Button6.FontName
  0xFFFF,               // Button6.Font_Color
  1,                    // Button6.FontHandle
  0,                    // Button6.Source
  1,                    // Button6.Active
  0,                    // Button6.OnUp
  0,                    // Button6.OnDown
  &Button6_OnClick,     // Button6.OnClick
  0                     // Button6.OnPress
};
TCEvent Button6_OnClick = {

  Button6OnClick,        // Button6_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button6_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button6_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button6_OnClick.Sound.Pitch
  255                    // Button6_OnClick.Sound.Volume
};
const code char Button6_Caption[2] = "C";
TCButton       Button7 = {
  &Calc_Demo,           // Button7.OwnerScreen
  7,                    // Button7.Order
  1,                    // Button7.Visible
  255,                  // Button7.Opacity
  255,                  // Button7.Tag
  1,                    // Button7.Left
  171,                  // Button7.Top
  95,                   // Button7.Width
  34,                   // Button7.Height
  1,                    // Button7.Pen_Width
  0x8410,               // Button7.Pen_Color
  0x39E7,               // Button7.Color
  0xFC08,               // Button7.Press_Color
  0xFFFF,               // Button7.ColorTo
  0xC618,               // Button7.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button7.Gradient
  Button7_Caption,      // Button7.Caption
  taCenter,             // Button7.TextAlign
  Tahoma_25x25_Bold,    // Button7.FontName
  0xFFFF,               // Button7.Font_Color
  1,                    // Button7.FontHandle
  0,                    // Button7.Source
  1,                    // Button7.Active
  0,                    // Button7.OnUp
  0,                    // Button7.OnDown
  &Button7_OnClick,     // Button7.OnClick
  0                     // Button7.OnPress
};
TCEvent Button7_OnClick = {

  Button7OnClick,        // Button7_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button7_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button7_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button7_OnClick.Sound.Pitch
  255                    // Button7_OnClick.Sound.Volume
};
const code char Button7_Caption[2] = "4";
TCButton       Button8 = {
  &Calc_Demo,           // Button8.OwnerScreen
  8,                    // Button8.Order
  1,                    // Button8.Visible
  255,                  // Button8.Opacity
  255,                  // Button8.Tag
  95,                   // Button8.Left
  171,                  // Button8.Top
  95,                   // Button8.Width
  34,                   // Button8.Height
  1,                    // Button8.Pen_Width
  0x8410,               // Button8.Pen_Color
  0x39E7,               // Button8.Color
  0xFC08,               // Button8.Press_Color
  0xFFFF,               // Button8.ColorTo
  0xC618,               // Button8.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button8.Gradient
  Button8_Caption,      // Button8.Caption
  taCenter,             // Button8.TextAlign
  Tahoma_25x25_Bold,    // Button8.FontName
  0xFFFF,               // Button8.Font_Color
  1,                    // Button8.FontHandle
  0,                    // Button8.Source
  1,                    // Button8.Active
  0,                    // Button8.OnUp
  0,                    // Button8.OnDown
  &Button8_OnClick,     // Button8.OnClick
  0                     // Button8.OnPress
};
TCEvent Button8_OnClick = {

  Button8OnClick,        // Button8_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button8_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button8_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button8_OnClick.Sound.Pitch
  255                    // Button8_OnClick.Sound.Volume
};
const code char Button8_Caption[2] = "5";
TCButton       Button9 = {
  &Calc_Demo,           // Button9.OwnerScreen
  9,                    // Button9.Order
  1,                    // Button9.Visible
  255,                  // Button9.Opacity
  255,                  // Button9.Tag
  285,                  // Button9.Left
  171,                  // Button9.Top
  97,                   // Button9.Width
  34,                   // Button9.Height
  1,                    // Button9.Pen_Width
  0x8410,               // Button9.Pen_Color
  0x39E7,               // Button9.Color
  0xFC08,               // Button9.Press_Color
  0xFFFF,               // Button9.ColorTo
  0xC618,               // Button9.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button9.Gradient
  Button9_Caption,      // Button9.Caption
  taCenter,             // Button9.TextAlign
  Tahoma_25x25_Bold,    // Button9.FontName
  0xFFFF,               // Button9.Font_Color
  1,                    // Button9.FontHandle
  0,                    // Button9.Source
  1,                    // Button9.Active
  0,                    // Button9.OnUp
  0,                    // Button9.OnDown
  &Button9_OnClick,     // Button9.OnClick
  0                     // Button9.OnPress
};
TCEvent Button9_OnClick = {

  Button9OnClick,        // Button9_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button9_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button9_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button9_OnClick.Sound.Pitch
  255                    // Button9_OnClick.Sound.Volume
};
const code char Button9_Caption[2] = "*";
TCButton       Button10 = {
  &Calc_Demo,           // Button10.OwnerScreen
  10,                   // Button10.Order
  1,                    // Button10.Visible
  255,                  // Button10.Opacity
  255,                  // Button10.Tag
  190,                  // Button10.Left
  171,                  // Button10.Top
  95,                   // Button10.Width
  34,                   // Button10.Height
  1,                    // Button10.Pen_Width
  0x8410,               // Button10.Pen_Color
  0x39E7,               // Button10.Color
  0xFC08,               // Button10.Press_Color
  0xFFFF,               // Button10.ColorTo
  0xC618,               // Button10.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button10.Gradient
  Button10_Caption,     // Button10.Caption
  taCenter,             // Button10.TextAlign
  Tahoma_25x25_Bold,    // Button10.FontName
  0xFFFF,               // Button10.Font_Color
  1,                    // Button10.FontHandle
  0,                    // Button10.Source
  1,                    // Button10.Active
  0,                    // Button10.OnUp
  0,                    // Button10.OnDown
  &Button10_OnClick,    // Button10.OnClick
  0                     // Button10.OnPress
};
TCEvent Button10_OnClick = {

  Button10OnClick,       // Button10_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button10_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button10_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button10_OnClick.Sound.Pitch
  255                    // Button10_OnClick.Sound.Volume
};
const code char Button10_Caption[2] = "6";
TCButton       Button11 = {
  &Calc_Demo,           // Button11.OwnerScreen
  11,                   // Button11.Order
  1,                    // Button11.Visible
  255,                  // Button11.Opacity
  255,                  // Button11.Tag
  382,                  // Button11.Left
  171,                  // Button11.Top
  98,                   // Button11.Width
  34,                   // Button11.Height
  1,                    // Button11.Pen_Width
  0x8410,               // Button11.Pen_Color
  0x39E7,               // Button11.Color
  0xFC08,               // Button11.Press_Color
  0xFFFF,               // Button11.ColorTo
  0xC618,               // Button11.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button11.Gradient
  Button11_Caption,     // Button11.Caption
  taCenter,             // Button11.TextAlign
  Tahoma_25x25_Bold,    // Button11.FontName
  0xFFFF,               // Button11.Font_Color
  1,                    // Button11.FontHandle
  0,                    // Button11.Source
  1,                    // Button11.Active
  0,                    // Button11.OnUp
  0,                    // Button11.OnDown
  &Button11_OnClick,    // Button11.OnClick
  0                     // Button11.OnPress
};
TCEvent Button11_OnClick = {

  Button11OnClick,       // Button11_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button11_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button11_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button11_OnClick.Sound.Pitch
  255                    // Button11_OnClick.Sound.Volume
};
const code char Button11_Caption[4] = "DEL";
TCButton       Button12 = {
  &Calc_Demo,           // Button12.OwnerScreen
  12,                   // Button12.Order
  1,                    // Button12.Visible
  255,                  // Button12.Opacity
  255,                  // Button12.Tag
  1,                    // Button12.Left
  205,                  // Button12.Top
  95,                   // Button12.Width
  34,                   // Button12.Height
  1,                    // Button12.Pen_Width
  0x8410,               // Button12.Pen_Color
  0x39E7,               // Button12.Color
  0xFC08,               // Button12.Press_Color
  0xFFFF,               // Button12.ColorTo
  0xC618,               // Button12.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button12.Gradient
  Button12_Caption,     // Button12.Caption
  taCenter,             // Button12.TextAlign
  Tahoma_25x25_Bold,    // Button12.FontName
  0xFFFF,               // Button12.Font_Color
  1,                    // Button12.FontHandle
  0,                    // Button12.Source
  1,                    // Button12.Active
  0,                    // Button12.OnUp
  0,                    // Button12.OnDown
  &Button12_OnClick,    // Button12.OnClick
  0                     // Button12.OnPress
};
TCEvent Button12_OnClick = {

  Button12OnClick,       // Button12_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button12_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button12_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button12_OnClick.Sound.Pitch
  255                    // Button12_OnClick.Sound.Volume
};
const code char Button12_Caption[2] = "1";
TCButton       Button13 = {
  &Calc_Demo,           // Button13.OwnerScreen
  13,                   // Button13.Order
  1,                    // Button13.Visible
  255,                  // Button13.Opacity
  255,                  // Button13.Tag
  95,                   // Button13.Left
  205,                  // Button13.Top
  95,                   // Button13.Width
  34,                   // Button13.Height
  1,                    // Button13.Pen_Width
  0x8410,               // Button13.Pen_Color
  0x39E7,               // Button13.Color
  0xFC08,               // Button13.Press_Color
  0xFFFF,               // Button13.ColorTo
  0xC618,               // Button13.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button13.Gradient
  Button13_Caption,     // Button13.Caption
  taCenter,             // Button13.TextAlign
  Tahoma_25x25_Bold,    // Button13.FontName
  0xFFFF,               // Button13.Font_Color
  1,                    // Button13.FontHandle
  0,                    // Button13.Source
  1,                    // Button13.Active
  0,                    // Button13.OnUp
  0,                    // Button13.OnDown
  &Button13_OnClick,    // Button13.OnClick
  0                     // Button13.OnPress
};
TCEvent Button13_OnClick = {

  Button13OnClick,       // Button13_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button13_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button13_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button13_OnClick.Sound.Pitch
  255                    // Button13_OnClick.Sound.Volume
};
const code char Button13_Caption[2] = "2";
TCButton       Button14 = {
  &Calc_Demo,           // Button14.OwnerScreen
  14,                   // Button14.Order
  1,                    // Button14.Visible
  255,                  // Button14.Opacity
  255,                  // Button14.Tag
  285,                  // Button14.Left
  205,                  // Button14.Top
  97,                   // Button14.Width
  34,                   // Button14.Height
  1,                    // Button14.Pen_Width
  0x8410,               // Button14.Pen_Color
  0x39E7,               // Button14.Color
  0xFC08,               // Button14.Press_Color
  0xFFFF,               // Button14.ColorTo
  0xC618,               // Button14.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button14.Gradient
  Button14_Caption,     // Button14.Caption
  taCenter,             // Button14.TextAlign
  Tahoma_25x25_Bold,    // Button14.FontName
  0xFFFF,               // Button14.Font_Color
  1,                    // Button14.FontHandle
  0,                    // Button14.Source
  1,                    // Button14.Active
  0,                    // Button14.OnUp
  0,                    // Button14.OnDown
  &Button14_OnClick,    // Button14.OnClick
  0                     // Button14.OnPress
};
TCEvent Button14_OnClick = {

  Button14OnClick,       // Button14_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button14_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button14_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button14_OnClick.Sound.Pitch
  255                    // Button14_OnClick.Sound.Volume
};
const code char Button14_Caption[2] = "-";
TCButton       Button15 = {
  &Calc_Demo,           // Button15.OwnerScreen
  15,                   // Button15.Order
  1,                    // Button15.Visible
  255,                  // Button15.Opacity
  255,                  // Button15.Tag
  190,                  // Button15.Left
  205,                  // Button15.Top
  95,                   // Button15.Width
  34,                   // Button15.Height
  1,                    // Button15.Pen_Width
  0x8410,               // Button15.Pen_Color
  0x39E7,               // Button15.Color
  0xFC08,               // Button15.Press_Color
  0xFFFF,               // Button15.ColorTo
  0xC618,               // Button15.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button15.Gradient
  Button15_Caption,     // Button15.Caption
  taCenter,             // Button15.TextAlign
  Tahoma_25x25_Bold,    // Button15.FontName
  0xFFFF,               // Button15.Font_Color
  1,                    // Button15.FontHandle
  0,                    // Button15.Source
  1,                    // Button15.Active
  0,                    // Button15.OnUp
  0,                    // Button15.OnDown
  &Button15_OnClick,    // Button15.OnClick
  0                     // Button15.OnPress
};
TCEvent Button15_OnClick = {

  Button15OnClick,       // Button15_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button15_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button15_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button15_OnClick.Sound.Pitch
  255                    // Button15_OnClick.Sound.Volume
};
const code char Button15_Caption[2] = "3";
TCButton       Button16 = {
  &Calc_Demo,           // Button16.OwnerScreen
  16,                   // Button16.Order
  1,                    // Button16.Visible
  255,                  // Button16.Opacity
  255,                  // Button16.Tag
  382,                  // Button16.Left
  205,                  // Button16.Top
  98,                   // Button16.Width
  67,                   // Button16.Height
  1,                    // Button16.Pen_Width
  0x8410,               // Button16.Pen_Color
  0x39E7,               // Button16.Color
  0xFC08,               // Button16.Press_Color
  0xFFFF,               // Button16.ColorTo
  0xC618,               // Button16.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button16.Gradient
  Button16_Caption,     // Button16.Caption
  taCenter,             // Button16.TextAlign
  Tahoma_25x25_Bold,    // Button16.FontName
  0xFFFF,               // Button16.Font_Color
  1,                    // Button16.FontHandle
  0,                    // Button16.Source
  1,                    // Button16.Active
  0,                    // Button16.OnUp
  0,                    // Button16.OnDown
  &Button16_OnClick,    // Button16.OnClick
  0                     // Button16.OnPress
};
TCEvent Button16_OnClick = {

  Button16OnClick,       // Button16_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button16_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button16_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button16_OnClick.Sound.Pitch
  255                    // Button16_OnClick.Sound.Volume
};
const code char Button16_Caption[2] = "=";
TCButton       Button17 = {
  &Calc_Demo,           // Button17.OwnerScreen
  17,                   // Button17.Order
  1,                    // Button17.Visible
  255,                  // Button17.Opacity
  255,                  // Button17.Tag
  1,                    // Button17.Left
  239,                  // Button17.Top
  189,                  // Button17.Width
  33,                   // Button17.Height
  1,                    // Button17.Pen_Width
  0x8410,               // Button17.Pen_Color
  0x39E7,               // Button17.Color
  0xFC08,               // Button17.Press_Color
  0xFFFF,               // Button17.ColorTo
  0xC618,               // Button17.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button17.Gradient
  Button17_Caption,     // Button17.Caption
  taCenter,             // Button17.TextAlign
  Tahoma_25x25_Bold,    // Button17.FontName
  0xFFFF,               // Button17.Font_Color
  1,                    // Button17.FontHandle
  0,                    // Button17.Source
  1,                    // Button17.Active
  0,                    // Button17.OnUp
  0,                    // Button17.OnDown
  &Button17_OnClick,    // Button17.OnClick
  0                     // Button17.OnPress
};
TCEvent Button17_OnClick = {

  Button17OnClick,       // Button17_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button17_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button17_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button17_OnClick.Sound.Pitch
  255                    // Button17_OnClick.Sound.Volume
};
const code char Button17_Caption[2] = "0";
TCButton       Button19 = {
  &Calc_Demo,           // Button19.OwnerScreen
  18,                   // Button19.Order
  1,                    // Button19.Visible
  255,                  // Button19.Opacity
  255,                  // Button19.Tag
  285,                  // Button19.Left
  239,                  // Button19.Top
  97,                   // Button19.Width
  33,                   // Button19.Height
  1,                    // Button19.Pen_Width
  0x8410,               // Button19.Pen_Color
  0x39E7,               // Button19.Color
  0xFC08,               // Button19.Press_Color
  0xFFFF,               // Button19.ColorTo
  0xC618,               // Button19.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button19.Gradient
  Button19_Caption,     // Button19.Caption
  taCenter,             // Button19.TextAlign
  Tahoma_25x25_Bold,    // Button19.FontName
  0xFFFF,               // Button19.Font_Color
  1,                    // Button19.FontHandle
  0,                    // Button19.Source
  1,                    // Button19.Active
  0,                    // Button19.OnUp
  0,                    // Button19.OnDown
  &Button19_OnClick,    // Button19.OnClick
  0                     // Button19.OnPress
};
TCEvent Button19_OnClick = {

  Button19OnClick,       // Button19_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button19_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button19_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button19_OnClick.Sound.Pitch
  255                    // Button19_OnClick.Sound.Volume
};
const code char Button19_Caption[2] = "+";
TCButton       Button20 = {
  &Calc_Demo,           // Button20.OwnerScreen
  19,                   // Button20.Order
  1,                    // Button20.Visible
  255,                  // Button20.Opacity
  255,                  // Button20.Tag
  190,                  // Button20.Left
  239,                  // Button20.Top
  95,                   // Button20.Width
  33,                   // Button20.Height
  1,                    // Button20.Pen_Width
  0x8410,               // Button20.Pen_Color
  0x39E7,               // Button20.Color
  0xFC08,               // Button20.Press_Color
  0xFFFF,               // Button20.ColorTo
  0xC618,               // Button20.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button20.Gradient
  Button20_Caption,     // Button20.Caption
  taCenter,             // Button20.TextAlign
  Tahoma_25x25_Bold,    // Button20.FontName
  0xFFFF,               // Button20.Font_Color
  1,                    // Button20.FontHandle
  0,                    // Button20.Source
  1,                    // Button20.Active
  0,                    // Button20.OnUp
  0,                    // Button20.OnDown
  &Button20_OnClick,    // Button20.OnClick
  0                     // Button20.OnPress
};
TCEvent Button20_OnClick = {

  Button20OnClick,       // Button20_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button20_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button20_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button20_OnClick.Sound.Pitch
  255                    // Button20_OnClick.Sound.Volume
};
const code char Button20_Caption[4] = "-/+";
TLabel         Label1;
char           Label1_Caption[20] = " ";
TLabel         Label2;
char           Label2_Caption[3] = " ";
TLabel         Label3;
char           Label3_Caption[20] = " ";
TLabel         Label4;
char           Label4_Caption[2] = "=";
TLabel         Label6;
char           Label6_Caption[20] = " ";
TCButton_Round ButtonRound1 = {
  &Calc_Demo,           // ButtonRound1.OwnerScreen
  25,                   // ButtonRound1.Order
  1,                    // ButtonRound1.Visible
  255,                  // ButtonRound1.Opacity
  255,                  // ButtonRound1.Tag
  434,                  // ButtonRound1.Left
  1,                    // ButtonRound1.Top
  40,                   // ButtonRound1.Width
  27,                   // ButtonRound1.Height
  1,                    // ButtonRound1.Pen_Width
  0x0000,               // ButtonRound1.Pen_Color
  0xF800,               // ButtonRound1.Color
  0xE71C,               // ButtonRound1.Press_Color
  ButtonRound1_Caption, // ButtonRound1.Caption
  taCenter,             // ButtonRound1.TextAlign
  Tahoma_19x23_Regular, // ButtonRound1.FontName
  0x0000,               // ButtonRound1.Font_Color
  2,                    // ButtonRound1.FontHandle
  12952,                // ButtonRound1.Source
  3,                    // ButtonRound1.Corner_Radius
  1,                    // ButtonRound1.Active
  0,                    // ButtonRound1.OnUp
  0,                    // ButtonRound1.OnDown
  &ButtonRound1_OnClick, // ButtonRound1.OnClick
  0                     // ButtonRound1.OnPress
};
TCEvent ButtonRound1_OnClick = {

  ButtonRound1OnClick,   // ButtonRound1_OnClick.Action
  VTFT_SNDACT_PLAY,      // ButtonRound1_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // ButtonRound1_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // ButtonRound1_OnClick.Sound.Pitch
  255                    // ButtonRound1_OnClick.Sound.Volume
};
const code char ButtonRound1_Caption[2] = "X";

TCButton *const code Calc_Demo_CButtons[19] = {
  &Button1,             
  &Button2,             
  &Button3,             
  &Button4,             
  &Button5,             
  &Button6,             
  &Button7,             
  &Button8,             
  &Button9,             
  &Button10,            
  &Button11,            
  &Button12,            
  &Button13,            
  &Button14,            
  &Button15,            
  &Button16,            
  &Button17,            
  &Button19,            
  &Button20             
};

TCButton_Round *const code Calc_Demo_CButtons_Round[1] = {
  &ButtonRound1         
};

TLabel *const code Calc_Demo_Labels[5] = {
  &Label1,              
  &Label2,              
  &Label3,              
  &Label4,              
  &Label6               
};

TCBox *const code Calc_Demo_CBoxes[1] = {
  &Box1                 
};


TScreen Main_Screen;

TCLabel  Label5 = {
  &Main_Screen,      // Label5.OwnerScreen
  0,                 // Label5.Order
  1,                 // Label5.Visible
  255,               // Label5.Opacity
  255,               // Label5.Tag
  103,               // Label5.Left
  27,                // Label5.Top
  286,               // Label5.Width
  28,                // Label5.Height
  Label5_Caption,    // Label5.Caption
  Tahoma_25x25_Bold, // Label5.FontName
  0x07FF,            // Label5.Font_Color
  2,                 // Label5.FontHandle
  8984,              // Label5.Source
  1,                 // Label5.Active
  0,                 // Label5.OnUp
  0,                 // Label5.OnDown
  0,                 // Label5.OnClick
  0                  // Label5.OnPress
};
const code char Label5_Caption[24] = "Calculator Demo Example";
TCLabel  Label7 = {
  &Main_Screen,         // Label7.OwnerScreen
  1,                    // Label7.Order
  1,                    // Label7.Visible
  255,                  // Label7.Opacity
  255,                  // Label7.Tag
  36,                   // Label7.Left
  75,                   // Label7.Top
  406,                  // Label7.Width
  25,                   // Label7.Height
  Label7_Caption,       // Label7.Caption
  Tahoma_19x23_Regular, // Label7.FontName
  0xFFFF,               // Label7.Font_Color
  1,                    // Label7.FontHandle
  0,                    // Label7.Source
  1,                    // Label7.Active
  0,                    // Label7.OnUp
  0,                    // Label7.OnDown
  0,                    // Label7.OnClick
  0                     // Label7.OnPress
};
const code char Label7_Caption[50] = "This is a simple version of calculator. Feel free";
TCLabel  Label8 = {
  &Main_Screen,         // Label8.OwnerScreen
  2,                    // Label8.Order
  1,                    // Label8.Visible
  255,                  // Label8.Opacity
  255,                  // Label8.Tag
  37,                   // Label8.Left
  103,                  // Label8.Top
  397,                  // Label8.Width
  25,                   // Label8.Height
  Label8_Caption,       // Label8.Caption
  Tahoma_19x23_Regular, // Label8.FontName
  0xFFFF,               // Label8.Font_Color
  1,                    // Label8.FontHandle
  0,                    // Label8.Source
  1,                    // Label8.Active
  0,                    // Label8.OnUp
  0,                    // Label8.OnDown
  0,                    // Label8.OnClick
  0                     // Label8.OnPress
};
const code char Label8_Caption[48] = "to try it out. Note that input values should be";
TCLabel  Label9 = {
  &Main_Screen,         // Label9.OwnerScreen
  3,                    // Label9.Order
  1,                    // Label9.Visible
  255,                  // Label9.Opacity
  255,                  // Label9.Tag
  31,                   // Label9.Left
  132,                  // Label9.Top
  432,                  // Label9.Width
  25,                   // Label9.Height
  Label9_Caption,       // Label9.Caption
  Tahoma_19x23_Regular, // Label9.FontName
  0xFFFF,               // Label9.Font_Color
  1,                    // Label9.FontHandle
  0,                    // Label9.Source
  1,                    // Label9.Active
  0,                    // Label9.OnUp
  0,                    // Label9.OnDown
  0,                    // Label9.OnClick
  0                     // Label9.OnPress
};
const code char Label9_Caption[50] = "within int/integer values (from -32768 to 32767).";
TCButton Button18 = {
  &Main_Screen,         // Button18.OwnerScreen
  4,                    // Button18.Order
  1,                    // Button18.Visible
  255,                  // Button18.Opacity
  255,                  // Button18.Tag
  284,                  // Button18.Left
  215,                  // Button18.Top
  176,                  // Button18.Width
  40,                   // Button18.Height
  1,                    // Button18.Pen_Width
  0x0000,               // Button18.Pen_Color
  0xC618,               // Button18.Color
  0xC618,               // Button18.Press_Color
  0xC618,               // Button18.ColorTo
  0xC618,               // Button18.Press_ColorTo
  _FT810_BRUSH_GR_NONE, // Button18.Gradient
  Button18_Caption,     // Button18.Caption
  taCenter,             // Button18.TextAlign
  Tahoma_19x23_Regular, // Button18.FontName
  0x0000,               // Button18.Font_Color
  1,                    // Button18.FontHandle
  0,                    // Button18.Source
  1,                    // Button18.Active
  0,                    // Button18.OnUp
  0,                    // Button18.OnDown
  &Button18_OnClick,    // Button18.OnClick
  0                     // Button18.OnPress
};
TCEvent Button18_OnClick = {

  Button18OnClick,       // Button18_OnClick.Action
  VTFT_SNDACT_PLAY,      // Button18_OnClick.Sound.SndAct
  _FT810_SOUND_CLICK,    // Button18_OnClick.Sound.Effect
  _FT810_SOUND_PITCH_A5, // Button18_OnClick.Sound.Pitch
  255                    // Button18_OnClick.Sound.Volume
};
const code char Button18_Caption[17] = "Continue to Demo";

TCButton *const code Main_Screen_CButtons[1] = {
  &Button18             
};

TCLabel *const code Main_Screen_CLabels[4] = {
  &Label5,              
  &Label7,              
  &Label8,              
  &Label9               
};


static char IsInsideObject(TObjInfo *AObjInfo, unsigned int X, unsigned int Y) {
  TRect  *ptrPressRect = 0;
  TRect  *ptrPressCircle = 0;
  TCRect *ptrPressRectC = 0;
  TCRect *ptrPressCircleC = 0;

  if ((AObjInfo->HitX == X) && (AObjInfo->HitY == Y))
    return 1;

  switch (AObjInfo->Type) {
    // CButton
    case VTFT_OT_CBUTTON: {
      ptrPressRectC = (TCRect *)&(((TCButton *)AObjInfo->Obj)->Left);
      break;
    }
    // CButtonRound
    case VTFT_OT_CBUTTONROUND: {
      ptrPressRectC = (TCRect *)&(((TCButton_Round *)AObjInfo->Obj)->Left);
      break;
    }
    // Label
    case VTFT_OT_LABEL: {
      ptrPressRect = (TRect *)&(((TLabel *)AObjInfo->Obj)->Left);
      break;
    }
    // CLabel
    case VTFT_OT_CLABEL: {
      ptrPressRectC = (TCRect *)&(((TCLabel *)AObjInfo->Obj)->Left);
      break;
    }
    // CBox
    case VTFT_OT_CBOX: {
      ptrPressRectC = (TCRect *)&(((TCBox *)AObjInfo->Obj)->Left);
      break;
    }
  }

  if (ptrPressRect) {
    if ((ptrPressRect->Left <= X) && (ptrPressRect->Left+ptrPressRect->Width-1 >= X) &&
        (ptrPressRect->Top  <= Y) && (ptrPressRect->Top+ptrPressRect->Height-1 >= Y))
      return 1;
    }
  else if (ptrPressRectC) {
    if ((ptrPressRectC->Left <= X) && (ptrPressRectC->Left+ptrPressRectC->Width-1 >= X) &&
        (ptrPressRectC->Top  <= Y) && (ptrPressRectC->Top +ptrPressRectC->Height-1 >= Y))
      return 1;
    }
   else if (ptrPressCircle) {
    if ((ptrPressCircle->Left <= X) && (ptrPressCircle->Left+ptrPressCircle->Width*2-1 >= X) &&
        (ptrPressCircle->Top  <= Y) && (ptrPressCircle->Top+ptrPressCircle->Width*2-1 >= Y))
      return 1;
    }
   else if (ptrPressCircleC) {
    if ((ptrPressCircleC->Left <= X) && (ptrPressCircleC->Left+ptrPressCircleC->Width*2-1 >= X) &&
        (ptrPressCircleC->Top  <= Y) && (ptrPressCircleC->Top+ptrPressCircleC->Width*2-1 >= Y))
      return 1;
    }

  return 0;
}

void DrawCButton(TCButton *ACButton) {
  int textWidth;
  int textHeight;
  
  if (ACButton->Visible) {
    if ((VTFT_OT_CBUTTON == TouchS.ActObjInfo.Type) && (ACButton == (TCButton *)TouchS.ActObjInfo.Obj)) {
      if (ACButton->Gradient != _FT810_BRUSH_GR_NONE) {
        FT810_Canvas_BrushGradient(_FT810_BRUSH_STYLE_SOLID, ACButton->Gradient, ACButton->Press_Color, ACButton->Press_ColorTo, ACButton->Opacity);
      }
      else {
        FT810_Canvas_BrushSingleColor(_FT810_BRUSH_STYLE_SOLID, ACButton->Press_Color, ACButton->Opacity);
      }
    }
    else {
      if (ACButton->Gradient != _FT810_BRUSH_GR_NONE) {
        FT810_Canvas_BrushGradient(_FT810_BRUSH_STYLE_SOLID, ACButton->Gradient, ACButton->Color, ACButton->ColorTo, ACButton->Opacity);
      }
      else {
        FT810_Canvas_BrushSingleColor(_FT810_BRUSH_STYLE_SOLID, ACButton->Color, ACButton->Opacity);
      }
    }

    FT810_Canvas_Pen(ACButton->Pen_Width, ACButton->Pen_Color, ACButton->Opacity);

    if (ACButton->Tag)
      FT810_Canvas_Tag(ACButton->Tag);

    FT810_Screen_Box(ACButton->Left, ACButton->Top, ACButton->Left+ACButton->Width-1, ACButton->Top+ACButton->Height-1);
    if (ACButton->FontHandle >= 16)
      FT810_Canvas_FontSystem(ACButton->FontHandle, ACButton->Font_Color, ACButton->Opacity);
    else
      FT810_Canvas_Font(ACButton->FontHandle, ACButton->FontName, ACButton->Source, ACButton->Font_Color, ACButton->Opacity);
    
    textWidth  = FT810_Canvas_TextWidth_Const(ACButton->Caption);
    textHeight = FT810_Canvas_TextHeight();
    if (ACButton->TextAlign == taLeft)
      FT810_Screen_TextPos_Const(ACButton->Left+(ACButton->Pen_Width)+1, ACButton->Top+(ACButton->Height-textHeight)/2, ACButton->Caption);
    else if (ACButton->TextAlign == taCenter)
      FT810_Screen_TextPos_Const(ACButton->Left+(ACButton->Width-textWidth)/2, ACButton->Top+(ACButton->Height-textHeight)/2, ACButton->Caption);
    else if (ACButton->TextAlign == taRight)
      FT810_Screen_TextPos_Const(ACButton->Left+ACButton->Width-textWidth-1, ACButton->Top+(ACButton->Height-textHeight)/2, ACButton->Caption);
  }
}

void DrawCButtonRound(TCButton_Round *ACButtonRound) {
  int textWidth;
  int textHeight;
  
  if (ACButtonRound->Visible) {
    if ((VTFT_OT_CBUTTONROUND == TouchS.ActObjInfo.Type) && (ACButtonRound == (TCButton_Round *)TouchS.ActObjInfo.Obj))
      FT810_Canvas_BrushSingleColor(_FT810_BRUSH_STYLE_SOLID, ACButtonRound->Press_Color, ACButtonRound->Opacity);
    else 
      FT810_Canvas_BrushSingleColor(_FT810_BRUSH_STYLE_SOLID, ACButtonRound->Color, ACButtonRound->Opacity);

    FT810_Canvas_Pen(ACButtonRound->Pen_Width, ACButtonRound->Pen_Color, ACButtonRound->Opacity);

    if (ACButtonRound->Tag)
      FT810_Canvas_Tag(ACButtonRound->Tag);

    FT810_Screen_BoxRound(ACButtonRound->Left, ACButtonRound->Top, ACButtonRound->Left+ACButtonRound->Width-1, ACButtonRound->Top+ACButtonRound->Height-1, ACButtonRound->Corner_Radius);
    if (ACButtonRound->FontHandle >= 16)
      FT810_Canvas_FontSystem(ACButtonRound->FontHandle, ACButtonRound->Font_Color, ACButtonRound->Opacity);
    else
      FT810_Canvas_Font(ACButtonRound->FontHandle, ACButtonRound->FontName, ACButtonRound->Source, ACButtonRound->Font_Color, ACButtonRound->Opacity);
    
    textWidth  = FT810_Canvas_TextWidth_Const(ACButtonRound->Caption);
    textHeight = FT810_Canvas_TextHeight();
    if (ACButtonRound->TextAlign == taLeft)
      FT810_Screen_TextPos_Const(ACButtonRound->Left+(ACButtonRound->Pen_Width)+1, ACButtonRound->Top+(ACButtonRound->Height-textHeight)/2, ACButtonRound->Caption);
    else if (ACButtonRound->TextAlign == taCenter)
      FT810_Screen_TextPos_Const(ACButtonRound->Left+(ACButtonRound->Width-textWidth)/2, ACButtonRound->Top+(ACButtonRound->Height-textHeight)/2, ACButtonRound->Caption);
    else if (ACButtonRound->TextAlign == taRight)
      FT810_Screen_TextPos_Const(ACButtonRound->Left+ACButtonRound->Width-textWidth-1, ACButtonRound->Top+(ACButtonRound->Height-textHeight)/2, ACButtonRound->Caption);
  }
}

void DrawLabel(TLabel *ALabel) {
  if (ALabel->Visible) {
    if (ALabel->FontHandle >= 16)
      FT810_Canvas_FontSystem(ALabel->FontHandle, ALabel->Font_Color, ALabel->Opacity);
    else
      FT810_Canvas_Font(ALabel->FontHandle, ALabel->FontName, ALabel->Source, ALabel->Font_Color, ALabel->Opacity);

    if (ALabel->Tag)
      FT810_Canvas_Tag(ALabel->Tag);

    FT810_Screen_TextPos(ALabel->Left, ALabel->Top, ALabel->Caption);
  }
}

void DrawCLabel(TCLabel *ACLabel) {
  if (ACLabel->Visible) {
    if (ACLabel->FontHandle >= 16)
      FT810_Canvas_FontSystem(ACLabel->FontHandle, ACLabel->Font_Color, ACLabel->Opacity);
    else
      FT810_Canvas_Font(ACLabel->FontHandle, ACLabel->FontName, ACLabel->Source, ACLabel->Font_Color, ACLabel->Opacity);

    if (ACLabel->Tag)
      FT810_Canvas_Tag(ACLabel->Tag);

    FT810_Screen_TextPos_Const(ACLabel->Left, ACLabel->Top, ACLabel->Caption);
  }
}

void DrawCBox(TCBox *ACBox) {
  if (ACBox->Visible) {
    if ((VTFT_OT_CBOX == TouchS.ActObjInfo.Type) && (ACBox == (TCBox *)TouchS.ActObjInfo.Obj)) {
      if (ACBox->Gradient != _FT810_BRUSH_GR_NONE) {
        FT810_Canvas_BrushGradient(_FT810_BRUSH_STYLE_SOLID, ACBox->Gradient, ACBox->Press_Color, ACBox->Press_ColorTo, ACBox->Opacity);
      }
      else {
        FT810_Canvas_BrushSingleColor(_FT810_BRUSH_STYLE_SOLID, ACBox->Press_Color, ACBox->Opacity);
      }
    }
    else {
      if (ACBox->Gradient != _FT810_BRUSH_GR_NONE) {
        FT810_Canvas_BrushGradient(_FT810_BRUSH_STYLE_SOLID, ACBox->Gradient, ACBox->Color, ACBox->ColorTo, ACBox->Opacity);
      }
      else {
        FT810_Canvas_BrushSingleColor(_FT810_BRUSH_STYLE_SOLID, ACBox->Color, ACBox->Opacity);
      }
    }

    FT810_Canvas_Pen(ACBox->Pen_Width, ACBox->Pen_Color, ACBox->Opacity);

    if (ACBox->Tag)
      FT810_Canvas_Tag(ACBox->Tag);

    FT810_Screen_Box(ACBox->Left, ACBox->Top, ACBox->Left+ACBox->Width-1, ACBox->Top+ACBox->Height-1);
  }
}

void SetIdenticalFontSources(const code char *AFontName, unsigned long ASource) {
  char i;
  TLabel *pLabel;
  const code void *const code *ptrCO;
  void *const code *ptrO;

  // Label
  i    = CurrentScreen->LabelsCount;
  ptrO = CurrentScreen->Labels;
  while (i--) {
    pLabel = (TLabel *)(*ptrO);
    if (pLabel->FontName == AFontName)
      pLabel->Source = ASource;
    ptrO++;
  }
}

static void ClearDynObjSource() {
  char i;
  TLabel *pLabel;
  const code void *const code *ptrCO;
  void *const code *ptrO;

  // Label
  i    = CurrentScreen->LabelsCount;
  ptrO = CurrentScreen->Labels;
  while (i--) {
    pLabel = (TLabel *)(*ptrO);
    pLabel->Source = 0xFFFFFFFF;
    ptrO++;
  }
}

void LoadCurrentScreenResToGRAM(char loadOptions) {
  char i;
  long currSource = -1;
  long tmpDynResStart;
  TCButton       *pCButton;
  TCButton_Round *pCButtonRound;
  TLabel         *pLabel;
  TCLabel        *pCLabel;
  const code void *const code *ptrCO;
  void *const code *ptrO;

  // clear dynamic resource addresses first, if necessary
  if (loadOptions & VTFT_LOAD_RES_DYNAMIC)
    ClearDynObjSource();

  // static resources are allocated by VTFT tool,
  // starting from address 0 in G Ram
  if (loadOptions & VTFT_LOAD_RES_STATIC) {

    // CButton
    i    = CurrentScreen->CButtonsCount;
    ptrCO = CurrentScreen->CButtons;
    while (i--) {
      pCButton = (TCButton *)(*ptrCO);
      if (pCButton->FontHandle < 16)
        if (pCButton->Source > currSource) {
          currSource = FT810_RES_LoadFontAddr(pCButton->Source, pCButton->FontName);
          SetIdenticalFontSources(pCButton->FontName, currSource);
        }
      ptrCO++;
    }

    // CButtonRound
    i    = CurrentScreen->CButtons_RoundCount;
    ptrCO = CurrentScreen->CButtons_Round;
    while (i--) {
      pCButtonRound = (TCButton_Round *)(*ptrCO);
      if (pCButtonRound->FontHandle < 16)
        if (pCButtonRound->Source > currSource) {
          currSource = FT810_RES_LoadFontAddr(pCButtonRound->Source, pCButtonRound->FontName);
          SetIdenticalFontSources(pCButtonRound->FontName, currSource);
        }
      ptrCO++;
    }

    // CLabel
    i    = CurrentScreen->CLabelsCount;
    ptrCO = CurrentScreen->CLabels;
    while (i--) {
      pCLabel = (TCLabel *)(*ptrCO);
      if (pCLabel->FontHandle < 16)
        if (pCLabel->Source > currSource) {
          currSource = FT810_RES_LoadFontAddr(pCLabel->Source, pCLabel->FontName);
          SetIdenticalFontSources(pCLabel->FontName, currSource);
        }
      ptrCO++;
    }
  }

  // dynamic resources allocation
  if (loadOptions & VTFT_LOAD_RES_DYNAMIC) {

    tmpDynResStart = CurrentScreen->DynResStart;
    if (FT810_RES_BeginLoad(tmpDynResStart)) {
      return;
    }

    // Label
    i    = CurrentScreen->LabelsCount;
    ptrO = CurrentScreen->Labels;
    while (i--) {
      pLabel = (TLabel *)(*ptrO);
      if (pLabel->FontHandle < 16)
        if (pLabel->Source == 0xFFFFFFFF) {
          currSource = FT810_RES_LoadFont(pLabel->FontName);
          pLabel->Source = currSource;
          SetIdenticalFontSources(pLabel->FontName, currSource);
        }
      ptrO++;
    }

    FT810_RES_EndLoad();
  }
}

void DrawObject(TPointer aObj, char aObjType) {
  TDrawHandler drawHandler;

  drawHandler = DrawHandlerTable[aObjType];
  if (drawHandler)
    drawHandler(aObj);
}

void DrawScreenO(TScreen *aScreen, char aOptions) {
  unsigned short cOrder, saveOrder;
  signed   int   actObjOrder;
  unsigned short pwmDuty;
  // counter variables
  char cntCButton;
  char cntCButtonRound;
  char cntLabel;
  char cntCLabel;
  char cntCBox;
  // pointer variables
  TCButton       *const code *pCButton;
  TCButton_Round *const code *pCButtonRound;
  TLabel         *const code *pLabel;
  TCLabel        *const code *pCLabel;
  TCBox          *const code *pCBox;

  // process screen switching effects
  if (aOptions & VTFT_DISPLAY_EFF_LIGHTS_FADE) {
    FT810_PWM_Get(0, &pwmDuty);
    FT810_PWM_FadeOut(pwmDuty, 0, pwmDuty/32? pwmDuty/32 : 1, 1);
  }
  else if (aOptions & VTFT_DISPLAY_EFF_LIGHTS_OFF) {
    FT810_PWM_Get(0, &pwmDuty);
    FT810_PWM_Duty(0);
  }

  if (CurrentScreen != aScreen) {
  // clear active object when drawing to new screen
    memset(&TouchS.ActObjInfo, 0, sizeof(TObjInfo));
  }

  CurrentScreen = aScreen;

  LoadCurrentScreenResToGRAM(aOptions);

  // init counter variables
  cntCButton      = CurrentScreen->CButtonsCount;
  cntCButtonRound = CurrentScreen->CButtons_RoundCount;
  cntLabel        = CurrentScreen->LabelsCount;
  cntCLabel       = CurrentScreen->CLabelsCount;
  cntCBox         = CurrentScreen->CBoxesCount;
  // init pointer variables
  pCButton      = CurrentScreen->CButtons;
  pCButtonRound = CurrentScreen->CButtons_Round;
  pLabel        = CurrentScreen->Labels;
  pCLabel       = CurrentScreen->CLabels;
  pCBox         = CurrentScreen->CBoxes;

  FT810_Screen_BeginUpdateCP();

  if ((FT810_Controller.Display.Width != CurrentScreen->Width) ||
      (FT810_Controller.Display.Height != CurrentScreen->Height)) {
    FT810_Controller.Display.Width = CurrentScreen->Width;
    FT810_Controller.Display.Height = CurrentScreen->Height;
    FT810_Canvas_UnClip();

    if (FT810_Controller.Display.Width > FT810_Controller.Display.Height) {
      FT810_CP_CmdSetRotate(VTFT_FT810_CONFIG_DISPLAY.Rotate180 % 2);
    }
    else {
      FT810_CP_CmdSetRotate((VTFT_FT810_CONFIG_DISPLAY.Rotate180 % 2) + 2);
    }
  }

  FT810_Canvas_BrushSingleColor(_FT810_BRUSH_STYLE_SOLID, CurrentScreen->Color, 255);
  FT810_Canvas_Tag(0);
  FT810_Screen_Clear(_FT810_CLEAR_ALL);
  FT810_CP_CmdStop();

  actObjOrder = -1;
  if (TouchS.ActObjInfo.Obj)
    if (TouchS.ActObjInfo.Flags & VTFT_INT_BRING_TO_FRONT)
      actObjOrder = TouchS.ActObjInfo.Order;

  cOrder = 0;
  while (cOrder < CurrentScreen->ObjectsCount) {
    saveOrder = cOrder;
    if (pCButton) {
      while ((*pCButton)->Order == cOrder) {
        if (actObjOrder != cOrder) // draw pressed object at the end
          DrawCButton(*pCButton);
        cOrder++;
        pCButton++;
        cntCButton--;
        if (!cntCButton) {
          pCButton = 0;
          break;
        }
      }
      if (saveOrder != cOrder)
        continue;
    }

    if (pCButtonRound) {
      while ((*pCButtonRound)->Order == cOrder) {
        if (actObjOrder != cOrder) // draw pressed object at the end
          DrawCButtonRound(*pCButtonRound);
        cOrder++;
        pCButtonRound++;
        cntCButtonRound--;
        if (!cntCButtonRound) {
          pCButtonRound = 0;
          break;
        }
      }
      if (saveOrder != cOrder)
        continue;
    }

    if (pLabel) {
      while ((*pLabel)->Order == cOrder) {
        if (actObjOrder != cOrder) // draw pressed object at the end
          DrawLabel(*pLabel);
        cOrder++;
        pLabel++;
        cntLabel--;
        if (!cntLabel) {
          pLabel = 0;
          break;
        }
      }
      if (saveOrder != cOrder)
        continue;
    }

    if (pCLabel) {
      while ((*pCLabel)->Order == cOrder) {
        if (actObjOrder != cOrder) // draw pressed object at the end
          DrawCLabel(*pCLabel);
        cOrder++;
        pCLabel++;
        cntCLabel--;
        if (!cntCLabel) {
          pCLabel = 0;
          break;
        }
      }
      if (saveOrder != cOrder)
        continue;
    }

    if (pCBox) {
      while ((*pCBox)->Order == cOrder) {
        if (actObjOrder != cOrder) // draw pressed object at the end
          DrawCBox(*pCBox);
        cOrder++;
        pCBox++;
        cntCBox--;
        if (!cntCBox) {
          pCBox = 0;
          break;
        }
      }
      if (saveOrder != cOrder)
        continue;
    }

    cOrder++;
  }

  // draw pressed object now
  if (TouchS.ActObjInfo.Obj)
    DrawObject(TouchS.ActObjInfo.Obj, TouchS.ActObjInfo.Type);

  FT810_Screen_EndUpdate();
  FT810_Screen_Show();

  // process screen switching effects
  if (aOptions & VTFT_DISPLAY_EFF_LIGHTS_FADE) {
    FT810_PWM_FadeIn(0, pwmDuty, 1, 3);
  }
  else if (aOptions & VTFT_DISPLAY_EFF_LIGHTS_OFF) {
    FT810_PWM_Duty(pwmDuty);
  }

}

void DrawScreen(TScreen *aScreen) {
  if (aScreen != CurrentScreen)
    DrawScreenO(aScreen, VTFT_LOAD_RES_ALL | VTFT_DISPLAY_EFF_LIGHTS_FADE);
  else
    DrawScreenO(aScreen, VTFT_LOAD_RES_NONE);
}

char GetActiveObjectByXY(int X, int Y, TObjInfo *AObjInfo) {
  char i;
  int  hiOrder;
  TCButton       *pCButton;
  TCButton_Round *pCButtonRound;
  TLabel         *pLabel;
  TCLabel        *pCLabel;
  TCBox          *pCBox;
  const code void *const code *ptrCO;
  void *const code *ptrO;

  // clear current object info
  memset(AObjInfo, 0, sizeof(TObjInfo));

  // Find object with highest order at specified position.
  // Objects lists are sorted by object order ascending.
  hiOrder = -1;

  // CButton
  i    = CurrentScreen->CButtonsCount;
  ptrCO = CurrentScreen->CButtons+CurrentScreen->CButtonsCount-1;
  while (i--) {
    pCButton = (TCButton *)(*ptrCO);
    if (pCButton->Order < hiOrder)
      break;
    if (pCButton->Active) {
      if ((pCButton->Left <= X) && (pCButton->Left+pCButton->Width-1 >= X) &&
          (pCButton->Top  <= Y) && (pCButton->Top+pCButton->Height-1 >= Y)) {
        AObjInfo->Obj   = (TPointer)pCButton;
        AObjInfo->Type  = VTFT_OT_CBUTTON;
        AObjInfo->Order = pCButton->Order;
        AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
        if ((pCButton->Press_Color != pCButton->Color) ||
            ((pCButton->Gradient != _FT810_BRUSH_GR_NONE) &&
             (pCButton->Press_ColorTo != pCButton->ColorTo)))
          AObjInfo->Flags |= VTFT_INT_REPAINT_ON_DOWN | VTFT_INT_REPAINT_ON_UP;

        hiOrder         = pCButton->Order;

        break;
      }
    }
    ptrCO--;
  }

  // CButtonRound
  i    = CurrentScreen->CButtons_RoundCount;
  ptrCO = CurrentScreen->CButtons_Round+CurrentScreen->CButtons_RoundCount-1;
  while (i--) {
    pCButtonRound = (TCButton_Round *)(*ptrCO);
    if (pCButtonRound->Order < hiOrder)
      break;
    if (pCButtonRound->Active) {
      if ((pCButtonRound->Left <= X) && (pCButtonRound->Left+pCButtonRound->Width-1 >= X) &&
          (pCButtonRound->Top  <= Y) && (pCButtonRound->Top+pCButtonRound->Height-1 >= Y)) {
        AObjInfo->Obj   = (TPointer)pCButtonRound;
        AObjInfo->Type  = VTFT_OT_CBUTTONROUND;
        AObjInfo->Order = pCButtonRound->Order;
        AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
        if (pCButtonRound->Press_Color != pCButtonRound->Color)
          AObjInfo->Flags |= VTFT_INT_REPAINT_ON_DOWN | VTFT_INT_REPAINT_ON_UP;

        hiOrder         = pCButtonRound->Order;

        break;
      }
    }
    ptrCO--;
  }

  // Label
  i    = CurrentScreen->LabelsCount;
  ptrO = CurrentScreen->Labels+CurrentScreen->LabelsCount-1;
  while (i--) {
    pLabel = (TLabel *)(*ptrO);
    if (pLabel->Order < hiOrder)
      break;
    if (pLabel->Active) {
      if ((pLabel->Left <= X) && (pLabel->Left+pLabel->Width-1 >= X) &&
          (pLabel->Top  <= Y) && (pLabel->Top+pLabel->Height-1 >= Y)) {
        AObjInfo->Obj   = (TPointer)pLabel;
        AObjInfo->Type  = VTFT_OT_LABEL;
        AObjInfo->Order = pLabel->Order;
        AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;

        hiOrder         = pLabel->Order;

        break;
      }
    }
    ptrO--;
  }

  // CLabel
  i    = CurrentScreen->CLabelsCount;
  ptrCO = CurrentScreen->CLabels+CurrentScreen->CLabelsCount-1;
  while (i--) {
    pCLabel = (TCLabel *)(*ptrCO);
    if (pCLabel->Order < hiOrder)
      break;
    if (pCLabel->Active) {
      if ((pCLabel->Left <= X) && (pCLabel->Left+pCLabel->Width-1 >= X) &&
          (pCLabel->Top  <= Y) && (pCLabel->Top+pCLabel->Height-1 >= Y)) {
        AObjInfo->Obj   = (TPointer)pCLabel;
        AObjInfo->Type  = VTFT_OT_CLABEL;
        AObjInfo->Order = pCLabel->Order;
        AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;

        hiOrder         = pCLabel->Order;

        break;
      }
    }
    ptrCO--;
  }

  // CBox
  i    = CurrentScreen->CBoxesCount;
  ptrCO = CurrentScreen->CBoxes+CurrentScreen->CBoxesCount-1;
  while (i--) {
    pCBox = (TCBox *)(*ptrCO);
    if (pCBox->Order < hiOrder)
      break;
    if (pCBox->Active) {
      if ((pCBox->Left <= X) && (pCBox->Left+pCBox->Width-1 >= X) &&
          (pCBox->Top  <= Y) && (pCBox->Top+pCBox->Height-1 >= Y)) {
        AObjInfo->Obj   = (TPointer)pCBox;
        AObjInfo->Type  = VTFT_OT_CBOX;
        AObjInfo->Order = pCBox->Order;
        AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
        if ((pCBox->Press_Color != pCBox->Color) ||
            ((pCBox->Gradient != _FT810_BRUSH_GR_NONE) &&
             (pCBox->Press_ColorTo != pCBox->ColorTo)))
          AObjInfo->Flags |= VTFT_INT_REPAINT_ON_DOWN | VTFT_INT_REPAINT_ON_UP;

        hiOrder         = pCBox->Order;

        break;
      }
    }
    ptrCO--;
  }

  if (AObjInfo->Obj) {
    AObjInfo->HitX = X;
    AObjInfo->HitY = Y;
    return 1;
  }
  else {
    return 0;
  }
}

char GetActiveObjectByTag(char ATag, TObjInfo *AObjInfo) {
  char i;
  TCButton       *pCButton;
  TCButton_Round *pCButtonRound;
  TLabel         *pLabel;
  TCLabel        *pCLabel;
  TCBox          *pCBox;
  const code void *const code *ptrCO;
  void *const code *ptrO;

  // clear current object info
  memset(AObjInfo, 0, sizeof(TObjInfo));

  // Find object with specified tag value.

  // CButton
  i    = CurrentScreen->CButtonsCount;
  ptrCO = CurrentScreen->CButtons+CurrentScreen->CButtonsCount-1;
  while (i--) {
    pCButton = (TCButton *)(*ptrCO);
    if (pCButton->Tag == ATag) {
      if (pCButton->Active) {
        AObjInfo->Obj   = (TPointer)pCButton;
        AObjInfo->Type  = VTFT_OT_CBUTTON;
        AObjInfo->Order = pCButton->Order;
        AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
        if ((pCButton->Press_Color != pCButton->Color) ||
            ((pCButton->Gradient != _FT810_BRUSH_GR_NONE) &&
             (pCButton->Press_ColorTo != pCButton->ColorTo)))
          AObjInfo->Flags |= VTFT_INT_REPAINT_ON_DOWN | VTFT_INT_REPAINT_ON_UP;

      }
      break;
    }
    ptrCO--;
  }

  // CButtonRound
  i    = CurrentScreen->CButtons_RoundCount;
  ptrCO = CurrentScreen->CButtons_Round+CurrentScreen->CButtons_RoundCount-1;
  while (i--) {
    pCButtonRound = (TCButton_Round *)(*ptrCO);
    if (pCButtonRound->Tag == ATag) {
      if (pCButtonRound->Active) {
        AObjInfo->Obj   = (TPointer)pCButtonRound;
        AObjInfo->Type  = VTFT_OT_CBUTTONROUND;
        AObjInfo->Order = pCButtonRound->Order;
        AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
        if (pCButtonRound->Press_Color != pCButtonRound->Color)
          AObjInfo->Flags |= VTFT_INT_REPAINT_ON_DOWN | VTFT_INT_REPAINT_ON_UP;

      }
      break;
    }
    ptrCO--;
  }

  // Label
  i    = CurrentScreen->LabelsCount;
  ptrO = CurrentScreen->Labels+CurrentScreen->LabelsCount-1;
  while (i--) {
    pLabel = (TLabel *)(*ptrO);
    if (pLabel->Tag == ATag) {
      if (pLabel->Active) {
        AObjInfo->Obj   = (TPointer)pLabel;
        AObjInfo->Type  = VTFT_OT_LABEL;
        AObjInfo->Order = pLabel->Order;
        AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
      }
      break;
    }
    ptrO--;
  }

  // CLabel
  i    = CurrentScreen->CLabelsCount;
  ptrCO = CurrentScreen->CLabels+CurrentScreen->CLabelsCount-1;
  while (i--) {
    pCLabel = (TCLabel *)(*ptrCO);
    if (pCLabel->Tag == ATag) {
      if (pCLabel->Active) {
        AObjInfo->Obj   = (TPointer)pCLabel;
        AObjInfo->Type  = VTFT_OT_CLABEL;
        AObjInfo->Order = pCLabel->Order;
        AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
      }
      break;
    }
    ptrCO--;
  }

  // CBox
  i    = CurrentScreen->CBoxesCount;
  ptrCO = CurrentScreen->CBoxes+CurrentScreen->CBoxesCount-1;
  while (i--) {
    pCBox = (TCBox *)(*ptrCO);
    if (pCBox->Tag == ATag) {
      if (pCBox->Active) {
        AObjInfo->Obj   = (TPointer)pCBox;
        AObjInfo->Type  = VTFT_OT_CBOX;
        AObjInfo->Order = pCBox->Order;
        AObjInfo->Flags = VTFT_INT_BRING_TO_FRONT;
        if ((pCBox->Press_Color != pCBox->Color) ||
            ((pCBox->Gradient != _FT810_BRUSH_GR_NONE) &&
             (pCBox->Press_ColorTo != pCBox->ColorTo)))
          AObjInfo->Flags |= VTFT_INT_REPAINT_ON_DOWN | VTFT_INT_REPAINT_ON_UP;

      }
      break;
    }
    ptrCO--;
  }

  if (AObjInfo->Obj) {
    AObjInfo->HitTag = ATag;
    return 1;
  }
  else {
    return 0;
  }
}

static void ProcessEvent(TEvent *pEvent) {
  if (pEvent) {
    if (pEvent->Sound.SndAct == VTFT_SNDACT_PLAY) 
      FT810_Sound_SetAndPlay(pEvent->Sound.Effect, pEvent->Sound.Pitch, pEvent->Sound.Volume);
    else if (pEvent->Sound.SndAct == VTFT_SNDACT_STOP) 
      FT810_Sound_Stop();
    if (pEvent->Action) 
      pEvent->Action();
  }
}

static void ProcessCEvent(TCEvent *pEventC) {
  if (pEventC) {
    if (pEventC->Sound.SndAct == VTFT_SNDACT_PLAY) 
      FT810_Sound_SetAndPlay(pEventC->Sound.Effect, pEventC->Sound.Pitch, pEventC->Sound.Volume);
    else if (pEventC->Sound.SndAct == VTFT_SNDACT_STOP) 
      FT810_Sound_Stop();
    if (pEventC->Action) 
      pEventC->Action();
  }
}

static void OnEvent(TObjInfo *AObjInfo, char AEventType){
  TEvent **ppEvent;
  TEvent  *pEvent = 0;
  TCEvent *const code *ppEventC;
  TCEvent *pEventC = 0;

  switch (AObjInfo->Type) {
    // CButton
    case VTFT_OT_CBUTTON: {
      ppEventC = &(((TCButton *)AObjInfo->Obj)->OnUp);
      pEventC  = ppEventC[AEventType];
      break;
    }
    // CButtonRound
    case VTFT_OT_CBUTTONROUND: {
      ppEventC = &(((TCButton_Round *)AObjInfo->Obj)->OnUp);
      pEventC  = ppEventC[AEventType];
      break;
    }
    // Label
    case VTFT_OT_LABEL: {
      ppEvent = &(((TLabel *)AObjInfo->Obj)->OnUp);
      pEvent  = ppEvent[AEventType];
      break;
    }
    // CLabel
    case VTFT_OT_CLABEL: {
      ppEventC = &(((TCLabel *)AObjInfo->Obj)->OnUp);
      pEventC  = ppEventC[AEventType];
      break;
    }
    // CBox
    case VTFT_OT_CBOX: {
      ppEventC = &(((TCBox *)AObjInfo->Obj)->OnUp);
      pEventC  = ppEventC[AEventType];
      break;
    }
  } // end switch

  if (pEvent) {
    ProcessEvent(pEvent);
  }

  if (pEventC) {
    ProcessCEvent(pEventC);
  }
}

static void Process_TP_Press() {
  // Screen Event
  if (CurrentScreen->Active)
    if ((CurrentScreen->SniffObjectEvents) || (!TouchS.ActObjInfo.Obj))
      ProcessEvent(CurrentScreen->OnPress);

  // Object Event
  if (!TouchS.ActObjInfo.Obj)
    return;

  OnEvent(&TouchS.ActObjInfo, VTFT_EVT_PRESS);
}

static void Process_TP_Up() {
  char     isClick;
  TObjInfo actObj;

  // Screen Event
  if (CurrentScreen->Active)
    if ((CurrentScreen->SniffObjectEvents) || (!TouchS.ActObjInfo.Obj))
      ProcessEvent(CurrentScreen->OnUp);

  actObj = TouchS.ActObjInfo;
  // Cler active object info
  memset(&TouchS.ActObjInfo, 0, sizeof(TObjInfo));

  // Object Event
  if (!actObj.Obj)
    return;

  isClick = IsInsideObject(&actObj, TouchS.X, TouchS.Y);

  if (actObj.Flags & VTFT_INT_REPAINT_ON_UP)
    DrawScreen(CurrentScreen);

  OnEvent(&actObj, VTFT_EVT_UP);
  if (isClick)
    OnEvent(&actObj, VTFT_EVT_CLICK);
}

static void Process_TP_Down() {
  // Search for active object
  if (TouchS.Tag) {        // objects must not have zero for tag value
    if (TouchS.Tag != 255) // can not search objects by default tag value
      GetActiveObjectByTag(TouchS.Tag, &TouchS.ActObjInfo);
    if (!TouchS.ActObjInfo.Obj) // object not found by tag, search by coordinates
      GetActiveObjectByXY(TouchS.X, TouchS.Y, &TouchS.ActObjInfo);
  }

  // Screen Event
  if (CurrentScreen->Active)
    if ((CurrentScreen->SniffObjectEvents) || (!TouchS.ActObjInfo.Obj))
      ProcessEvent(CurrentScreen->OnDown);

  // Object Event
  if (!TouchS.ActObjInfo.Obj)
    return;

  if (TouchS.ActObjInfo.Flags & VTFT_INT_REPAINT_ON_DOWN)
    DrawScreen(CurrentScreen);

  OnEvent(&TouchS.ActObjInfo, VTFT_EVT_DOWN);
}

static void Process_TP_TagChange() {
  // Screen Event
  if (CurrentScreen->Active)
    ProcessEvent(CurrentScreen->OnTagChange);
}

void ProcessVTFTStack() {
  char         Tag, oldTag;
  unsigned int X, Y;

  oldTag = TouchS.Tag;

  if (FT810_Touch_GetTagXY(&X, &Y) == 1) {
    FT810_Touch_GetTag(&Tag);

    TouchS.Tag = Tag;
    TouchS.X = X;
    TouchS.Y = Y;

    if (!TouchS.Pressed) {
      TouchS.Pressed = 1;
      Process_TP_Down();
    }

    Process_TP_Press();
  }
  else if (TouchS.Pressed) {
    Process_TP_Up();

    TouchS.Tag = 0;
    TouchS.X   = X;
    TouchS.Y   = Y;

    TouchS.Pressed = 0;
  }

  if (oldTag != TouchS.Tag)
    Process_TP_TagChange();
}

static void InitObjects() {
  // Calc_Demo: Init block start
  Calc_Demo.Color               = 0x8410;
  Calc_Demo.Width               = 480;
  Calc_Demo.Height              = 272;
  Calc_Demo.ObjectsCount        = 26;
  Calc_Demo.CButtonsCount       = 19;
  Calc_Demo.CButtons            = Calc_Demo_CButtons;
  Calc_Demo.CButtons_RoundCount = 1;
  Calc_Demo.CButtons_Round      = Calc_Demo_CButtons_Round;
  Calc_Demo.LabelsCount         = 5;
  Calc_Demo.Labels              = Calc_Demo_Labels;
  Calc_Demo.CLabelsCount        = 0;
  Calc_Demo.CLabels             = 0;
  Calc_Demo.CBoxesCount         = 1;
  Calc_Demo.CBoxes              = Calc_Demo_CBoxes;
  Calc_Demo.DynResStart         = 21936;
  Calc_Demo.Active              = 1;
  Calc_Demo.SniffObjectEvents   = 0;
  Calc_Demo.OnUp                = 0;
  Calc_Demo.OnDown              = 0;
  Calc_Demo.OnTagChange         = 0;
  Calc_Demo.OnPress             = 0;

  // Main_Screen: Init block start
  Main_Screen.Color               = 0x0000;
  Main_Screen.Width               = 480;
  Main_Screen.Height              = 272;
  Main_Screen.ObjectsCount        = 5;
  Main_Screen.CButtonsCount       = 1;
  Main_Screen.CButtons            = Main_Screen_CButtons;
  Main_Screen.CButtons_RoundCount = 0;
  Main_Screen.CButtons_Round      = 0;
  Main_Screen.LabelsCount         = 0;
  Main_Screen.Labels              = 0;
  Main_Screen.CLabelsCount        = 4;
  Main_Screen.CLabels             = Main_Screen_CLabels;
  Main_Screen.CBoxesCount         = 0;
  Main_Screen.CBoxes              = 0;
  Main_Screen.DynResStart         = 21936;
  Main_Screen.Active              = 1;
  Main_Screen.SniffObjectEvents   = 0;
  Main_Screen.OnUp                = 0;
  Main_Screen.OnDown              = 0;
  Main_Screen.OnTagChange         = 0;
  Main_Screen.OnPress             = 0;

  Label1.OwnerScreen = &Calc_Demo;
  Label1.Order       = 20;
  Label1.Visible     = 1;
  Label1.Opacity     = 255;
  Label1.Tag         = 255;
  Label1.Left        = 15;
  Label1.Top         = 33;
  Label1.Width       = 0;
  Label1.Height      = 25;
  Label1.Caption     = Label1_Caption;
  Label1.FontName    = Tahoma_19x23_Regular;
  Label1.Font_Color  = 0x0000;
  Label1.FontHandle  = 2;
  Label1.Source      = -1UL;
  Label1.Active      = 1;
  Label1.OnUp        = 0;
  Label1.OnDown      = 0;
  Label1.OnClick     = 0;
  Label1.OnPress     = 0;

  Label2.OwnerScreen = &Calc_Demo;
  Label2.Order       = 21;
  Label2.Visible     = 1;
  Label2.Opacity     = 255;
  Label2.Tag         = 255;
  Label2.Left        = 15;
  Label2.Top         = 59;
  Label2.Width       = 0;
  Label2.Height      = 25;
  Label2.Caption     = Label2_Caption;
  Label2.FontName    = Tahoma_19x23_Regular;
  Label2.Font_Color  = 0x0000;
  Label2.FontHandle  = 2;
  Label2.Source      = -1UL;
  Label2.Active      = 1;
  Label2.OnUp        = 0;
  Label2.OnDown      = 0;
  Label2.OnClick     = 0;
  Label2.OnPress     = 0;

  Label3.OwnerScreen = &Calc_Demo;
  Label3.Order       = 22;
  Label3.Visible     = 1;
  Label3.Opacity     = 255;
  Label3.Tag         = 255;
  Label3.Left        = 15;
  Label3.Top         = 84;
  Label3.Width       = 0;
  Label3.Height      = 25;
  Label3.Caption     = Label3_Caption;
  Label3.FontName    = Tahoma_19x23_Regular;
  Label3.Font_Color  = 0x0000;
  Label3.FontHandle  = 2;
  Label3.Source      = -1UL;
  Label3.Active      = 1;
  Label3.OnUp        = 0;
  Label3.OnDown      = 0;
  Label3.OnClick     = 0;
  Label3.OnPress     = 0;

  Label4.OwnerScreen = &Calc_Demo;
  Label4.Order       = 23;
  Label4.Visible     = 1;
  Label4.Opacity     = 255;
  Label4.Tag         = 255;
  Label4.Left        = 15;
  Label4.Top         = 110;
  Label4.Width       = 14;
  Label4.Height      = 25;
  Label4.Caption     = Label4_Caption;
  Label4.FontName    = Tahoma_19x23_Regular;
  Label4.Font_Color  = 0x0000;
  Label4.FontHandle  = 2;
  Label4.Source      = -1UL;
  Label4.Active      = 1;
  Label4.OnUp        = 0;
  Label4.OnDown      = 0;
  Label4.OnClick     = 0;
  Label4.OnPress     = 0;

  Label6.OwnerScreen = &Calc_Demo;
  Label6.Order       = 24;
  Label6.Visible     = 1;
  Label6.Opacity     = 255;
  Label6.Tag         = 255;
  Label6.Left        = 38;
  Label6.Top         = 110;
  Label6.Width       = 0;
  Label6.Height      = 25;
  Label6.Caption     = Label6_Caption;
  Label6.FontName    = Tahoma_19x23_Regular;
  Label6.Font_Color  = 0x0000;
  Label6.FontHandle  = 2;
  Label6.Source      = -1UL;
  Label6.Active      = 1;
  Label6.OnUp        = 0;
  Label6.OnDown      = 0;
  Label6.OnClick     = 0;
  Label6.OnPress     = 0;

}

void Init_MCU(){
//    SPI2_Init_Advanced(_SPI_MASTER_OSC_DIV4, _SPI_DATA_SAMPLE_MIDDLE,
//                                    _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);
    ANCON1 = 0X00;
    ANCON2 = 0x00;
    ANCON3 = 0x00;


    RPOR18_19 = 0x05;
    TRISC.B6 = 0;

    RPOR16_17 = 0x04;
    TRISC.B5 = 0;

    RPINR12_13 = 0xF7;
    TRISE.B1 = 1;


    SPI2_Remappable_Init_Advanced( _SPI_MASTER_OSC_DIV4,
    _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH );
}

void InitVTFTStack() {
  union {
    TFT810PWM       PWM;
    TFT810GPIO      GPIO;
    TFT810Audio     Audio;
    TFT810Sound     Sound;
    TFT810Touch     Touch;
    TFT810Display   Display;
    TFT810Interrupt Interrupt;
  } cfg;

  Init_MCU();

  SPI_Set_Active(SPI2_Read, SPI2_Write);

  // Init all dynamic objects
  InitObjects();

  // Init FT810 controller core and library stack
  FT810_Init();

  FT810_Core_ClockSource(_FT810_CLK_SOURCE_EXTERNAL);
  FT810_Core_SetFrequency(VTFT_FT810_CONFIG_DISPLAY.Frequency);

  // Internal modules setup
  cfg.Display = VTFT_FT810_CONFIG_DISPLAY;
  FT810_Display_SetConfig(&cfg.Display);

  cfg.Audio = VTFT_FT810_CONFIG_AUDIO;
  FT810_Audio_SetConfig(&cfg.Audio);

  cfg.Sound = VTFT_FT810_CONFIG_SOUND;
  FT810_Sound_SetConfig(&cfg.Sound);

  cfg.Interrupt = VTFT_FT810_CONFIG_INTERRUPT;
  FT810_Interrupt_SetConfig(&cfg.Interrupt);

  cfg.PWM = VTFT_FT810_CONFIG_PWM;
  FT810_PWM_SetConfig(&cfg.PWM);

  cfg.GPIO = VTFT_FT810_CONFIG_GPIO;
  FT810_GPIO_SetConfig(&cfg.GPIO);

  cfg.Touch = VTFT_FT810_CONFIG_TOUCH;
  FT810_Touch_SetConfig(&cfg.Touch);

  FT810_Touch_Calibrate(_FT810_FONT_ROBOTO_SIZE_10, "Touch blinking point on the screen!", Calc_Demo.Width/2, Calc_Demo.Height/2);

  // Draw start screen
  DrawScreen(&Calc_Demo);
}